#! python
'''
UCL positronium spectroscopy blog, https://antimattergravity.com.

Usage:
    >>> from positronium import antimattergravity

Try also:
    >>> import antigravity
'''
import webbrowser

webbrowser.open("https://antimattergravity.com/")
